<?php
/*
* Plugin Name: Delivery Location Map
* Plugin URI:  
* Description: Optional Map to checkout page to get the exact delivery location
* Version: 1.0
* Author: 
* Author URI: 
* Text Domain: dlm
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) 
{
  exit;
}
if ( ! defined( 'DLM_VERSION' ) ) 
{
    define( 'DLM_VERSION', '1.0.0' );
}
if ( ! defined( 'DLM_CSS_URI' ) ) 
{
    define( 'DLM_CSS_URI', plugins_url( 'css/',__FILE__ ) );
}
if ( ! defined( 'DLM_JS_URI' ) ) 
{
    define( 'DLM_JS_URI', plugins_url( 'js/',__FILE__ ) );
}
if ( ! defined( 'DLM_IMAGE_URI' ) ) 
{
    define( 'DLM_IMAGE_URI', plugins_url( 'image/',__FILE__ ) );
}
class DLM
{
    private $api_key;
    private $delivery_title;
    private $default_zoom;
    private $default_lat_long;
    private $qr_code_size;
    private $map_width;
    private $map_height;
    function __construct()
    {   
        $this->qr_code='no';
        $this->qr_code_size='100x100';
        $this->default_lat_long='21.710216040289918,77.41453513183592';
        $this->delivery_title='Delivery Location';
        $this->default_zoom=10;
        $this->map_width='100%';
        $this->map_height='380px';
        $this->api_key='';//'AIzaSyAi0nz2wyFAGU2jy3x_WsEZ-OCYqWFeS5Q';
        add_action('wp_enqueue_scripts', array($this,'dlm_styles_scripts'),50);
        add_action('woocommerce_after_order_notes', array($this,'dlm_checkout_field'));
        add_action('woocommerce_checkout_process', array($this,'dlm_checkout_field_process'));
        add_action('woocommerce_checkout_create_order', array($this,'dlm_checkout_field_create_order'), 10, 2 );
        add_action('woocommerce_order_details_after_order_table', array($this,'dlm_custom_field_display_order_details'), 10, 1 );
        add_action('woocommerce_admin_order_data_after_shipping_address', array($this,'dlm_checkout_field_display_admin_order_meta'), 10, 1 );
        
        add_filter('woocommerce_settings_tabs_array', array($this,'dlm_add_settings_tab'), 50 );
        add_action('woocommerce_settings_tabs_dlm_api_keys', array($this,'dlm_settings_tab' ));
        add_action('woocommerce_update_options_dlm_api_keys', array($this,'dlm_update_settings' ));
        
        add_action('wpo_wcpdf_after_document_label', array($this,'dlm_wcpdf_qrcode'), 10, 2 );        
        add_filter( 'plugin_action_links', array($this,'dlm_links'), 10, 2 );
        add_action( 'wp_footer', array($this,'dlm_checkout_map_script' ));
        
        add_action( 'woocommerce_email_order_meta', array($this,'dlm_add_email_order_meta'), 10, 3 );
       
        /*** FOR LOCATION COLUMN IN LIST **/
        add_filter( 'manage_edit-shop_order_columns', array($this,'dlm_goole_map_column'), 10 );
        add_action( 'manage_shop_order_posts_custom_column', array($this,'dlm_google_map_column_content') );

    }//end of function
    function dlm_styles_scripts()
    {        
        $chosen_methods = WC()->session->get( 'chosen_shipping_methods' );
        $chosen_shipping = $chosen_methods[0];
        $chosen_shipping_arr=explode(':',$chosen_shipping);
        wp_enqueue_style( 'dlm-style-css', DLM_CSS_URI. 'dlm.css',array(),dlm_VERSION);     
        if(is_checkout())
        {           
            if(get_option('dlm_api_keys_google_map'))
            {
                $this->api_key=get_option('dlm_api_keys_google_map');
            }
            if(get_option('dlm_map_zoom'))
            {
                $this->default_zoom=get_option('dlm_map_zoom');
            }
            if(get_option('dlm_google_map_width'))
            {
                $this->map_width=get_option('dlm_google_map_width');
            }
            if(get_option('dlm_google_map_height'))
            {
                $this->map_height=get_option('dlm_google_map_height');
            }
            
            wp_enqueue_script( 'google-map', 'https://maps.googleapis.com/maps/api/js?key='.$this->api_key);
            wp_enqueue_script( 'location-map', DLM_JS_URI.'dlm-map.js');            
            wp_localize_script( 'location-map', 'zoom', $this->default_zoom ); 
            if(is_user_logged_in())
            {
                $user_id=get_current_user_id();
                $deliver_location=get_user_meta($user_id,'delivery_location',true);
                $latlong=explode(',',$deliver_location);
                if(is_array($latlong) && isset( $latlong[0]) && isset( $latlong[1]))
                {
                    wp_localize_script( 'location-map', 'lat', $latlong[0] ); 
                    wp_localize_script( 'location-map', 'long', $latlong[1] );                    
                }
            }
            else
            {
                if(get_option('dlm_delivery_default_loc'))
                {
                    $this->default_lat_long=get_option('dlm_delivery_default_loc');
                } 
                $latlong=explode(',',$this->default_lat_long);
                if(is_array($latlong) && isset( $latlong[0]) && isset( $latlong[1]))
                {
                    wp_localize_script( 'location-map', 'lat', $latlong[0] ); 
                    wp_localize_script( 'location-map', 'long', $latlong[1] );                    
                }
            }
        }    
    }//end of function
    function dlm_checkout_field($checkout)
    {        
        $chosen_methods = WC()->session->get( 'chosen_shipping_methods' );
        $chosen_shipping = $chosen_methods[0];
        $chosen_shipping_arr=explode(':',$chosen_shipping);
        $hide_class="";
        if ($chosen_shipping_arr[0] == 'local_pickup') 
        {
            $hide_class="hide";
        }
            $html='<div id="customise_checkout_field" class="'.$hide_class.'">';
                $html.='<h2>';
                if(get_option('dlm_delivery_location_title'))
                {
                    $this->delivery_title=get_option('dlm_delivery_location_title');
                }            
                $html.=esc_html__($this->delivery_title,'dlm');
                $html.='</h2>';
                if(get_option('dlm_delivery_location_short_sec'))
                {
                    $html.='<div class="delivery_short_desc">';
                        $delivery_short_desc=get_option('dlm_delivery_location_short_sec');
                        $html.=esc_html__($delivery_short_desc,'dlm');
                    $html.='</div>';
                }            
                ob_start();
                woocommerce_form_field('dlm_shipping_method', array(
                    'type' => 'text',
                    'class' => array(
                      'shipping-class form-row-wide'
                    ) ,
                ) , $chosen_shipping);
                woocommerce_form_field('delivery_location', array(
                    'type' => 'text',
                    'class' => array(
                      'latlong-class form-row-wide'
                    ) ,
                ) , $checkout->get_value('delivery_location'));
                $html.=ob_get_clean();
                if($this->api_key=='')
                {
                    $html.='<span class="dlm_error">To Get the working Google Map, Please set API key in Setting. WooCommerce -> Settings-> Delivery Location</span>';
                }
                $html.='<div id="dlmmapdiv" style="width:'.$this->map_width.';height:'.$this->map_height.';">';
                $html.='</div>';
            $html.='</div>';
            echo $html;
        
    }//end of function
    function dlm_checkout_field_process()
    {
        $chosen_methods = WC()->session->get( 'chosen_shipping_methods' );
        $chosen_shipping = $chosen_methods[0];
        $chosen_shipping_arr=explode(':',$chosen_shipping);
        if ($chosen_shipping_arr[0] != 'local_pickup')
        {
            // if the field is set, if not then show an error message.
            if (!$_POST['delivery_location']) wc_add_notice(__('Please enter value.') , 'error');
        }
    }//end of function
    function dlm_checkout_field_create_order($order,$data)
    {
        $order->update_meta_data( 'dlm_shipping_method', sanitize_text_field($_POST['dlm_shipping_method']));
        if (!empty($_POST['delivery_location'])) 
        {
            $order->update_meta_data( 'dlm_delivery_location', sanitize_text_field($_POST['delivery_location']));
            update_user_meta(get_current_user_id(),'delivery_location',sanitize_text_field($_POST['delivery_location']));
        }
    }//end of function
    function dlm_custom_field_display_order_details($order)
    { 
        $this->qr_code=get_option('dlm_qr_code_yn');        
        $this->qr_code_size=get_option('dlm_qr_code_size');
        $chosen_shipping =  get_post_meta( $order->id, 'dlm_shipping_method', true );
        $chosen_shipping_arr=explode(':',$chosen_shipping);
        if ($chosen_shipping_arr[0] != 'local_pickup')
        {
            $delivery_url=$this->dlm_delivery_url($order);
            $html='<p>';
                $html.='<strong>';
                $html.=esc_html__('Deliver Location URL');
                $html.=':</strong> ';
                $html.='<a href="'.esc_url($delivery_url).'">'.esc_html__('Google Map','dlm').'</a>';
            $html.='</p>';
            if($this->qr_code=='yes')
            {
                $html.='<p>';
                    $html.='<strong>';
                    $html.=esc_html__('Deliver Location QR Code');
                    $html.=':</strong>';
                    $html.='<br/><img src="'.DLM_IMAGE_URI.'qrcode.php?code='.$delivery_url.'&size='.$this->qr_code_size.'">';
                $html.='</p>';
            }    
            echo $html;
        }
    }//end of function
    function dlm_delivery_url($order)
    {
        return $delivery_url='http://maps.google.com/?q='.get_post_meta( $order->id, 'dlm_delivery_location', true );
    }//end of function  
    function dlm_checkout_field_display_admin_order_meta($order)
    {
        $this->qr_code=get_option('dlm_qr_code_yn');
        $this->qr_code_size=get_option('dlm_qr_code_size');        
        $chosen_shipping =  get_post_meta( $order->id, 'dlm_shipping_method', true );
        $chosen_shipping_arr=explode(':',$chosen_shipping);
        if ($chosen_shipping_arr[0] != 'local_pickup')
        {
            $delivery_url=$this->dlm_delivery_url($order);
            $html='<p>';
                $html.='<strong>';
                $html.=esc_html__('Deliver Location URL');
                $html.=':</strong> ';
                $html.='<a href="'.esc_url($delivery_url).'">'.esc_html__('Google Map','dlm').'</a>';
            $html.='</p>';
            if($this->qr_code=='yes')
            {
                $html.='<p>';
                    $html.='<strong>';
                    $html.=esc_html__('Deliver Location QR Code');
                    $html.=':</strong>';
                    $html.='<br/><img src="'.DLM_IMAGE_URI.'qrcode.php?code='.$delivery_url.'&size='.$this->qr_code_size.'">';
                $html.='</p>';
            }            
            echo $html;
        }
    }//end of function
    function dlm_add_settings_tab( $settings_tabs ) 
    {
        $settings_tabs['dlm_api_keys'] = esc_html__( 'Delivery Location', 'dlm' );
        return $settings_tabs;
    }//end of function
    function dlm_settings_tab() 
    {
        woocommerce_admin_fields( $this->dlm_get_settings() );
    }//end of function
    function dlm_get_settings() 
    {
        $settings = array(

                'dlm_section_title' => array(
                    'name'     => esc_html__( '', 'dlm' ),
                    'type'     => 'title',
                    'desc'     => '',
                    'id'       => 'dlm_section_starts'
                ),
                'dlm_delivery_title' => array(
                    'name' => esc_html__( 'Delivery Location Title', 'dlm' ),
                    'type' => 'text',
                    'id'   => 'dlm_delivery_location_title'
                ), 
                'dlm_delivery_short_desc' => array(
                    'name' => esc_html__( 'Short Description', 'dlm' ),
                    'type' => 'textarea',
                    'id'   => 'dlm_delivery_location_short_sec'
                ),
                // 'dlm_qr_code_yn' => array(
                //     'name' => esc_html__( 'QR Code ( Yes/No )', 'dlm' ),
                //     'type' => 'checkbox',
                //     'desc'     => 'Do you need qrcode to be added',
                //     'id'   => 'dlm_qr_code_yn'
                // ),
                // 'dlm_qr_code_size' => array(
                //     'name' => esc_html__( 'QR Code Size', 'dlm' ),
                //     'type' => 'text',
                //     'desc'     => '100x100 will be the format',
                //     'placeholder'=>'100x100',
                //     'id'   => 'dlm_qr_code_size'
                // ),
                'dlm_google_map_width' => array(
                    'name' => esc_html__( 'Google Map Width', 'dlm' ),
                    'type' => 'text',
                    'desc'     => '[Value with unit e.g 100% or 200px ]',
                    'placeholder'=>'100%',
                    'id'   => 'dlm_google_map_width'
                ),
                'dlm_google_map_height' => array(
                    'name' => esc_html__( 'Google Map Height', 'dlm' ),
                    'type' => 'text',
                    'desc'     => '[Value with unit e.g 100% or 200px ]',
                    'placeholder'=>'300px',
                    'id'   => 'dlm_google_map_height'
                ),
                'dlm_delivery_map_zoom' => array(
                    'name' => esc_html__( 'Map Zoom', 'dlm' ),
                    'type' => 'select',
                    'id'   => 'dlm_map_zoom',
                    'options' => array(
                              '5' => __( '5' ),
                              '6' => __( '6' ),
                              '7' => __( '7' ),
                              '8' => __( '8' ),
                              '9' => __( '9' ),
                              '10' => __( '10' ),
                              '11' => __( '11' ),
                              '12' => __( '12' ),
                              '13' => __( '13' ),
                              '14' => __( '14' ),
                              '15' => __( '15' ),
                              '16' => __( '16' ),
                    )
                ),   
                'dlm_map_api_key' => array(
                    'name' => esc_html__( 'Google Map API Key', 'dlm' ),
                    'type' => 'text',
                    'desc' => __( '<a href="https://developers.google.com/maps/documentation/javascript/get-api-key" style="text-decoration:none;"><span class="dashicons dashicons-editor-help"></span></a>', 'dlm' ),
                    'id'   => 'dlm_api_keys_google_map'
                ),            
                'dlm_section_end' => array(
                     'type' => 'sectionend',
                     'id' => 'dlm_section_end'
                )
            );
        return apply_filters( 'wc_dlm_api_keys_settings', $settings );
    }//end of function
    function dlm_update_settings() 
    {
        woocommerce_update_options( $this->dlm_get_settings());
    }//end of function
    function dlm_wcpdf_qrcode ($template_type, $order) 
    {
        $chosen_shipping =  get_post_meta( $order->id, 'dlm_shipping_method', true );
        $chosen_shipping_arr=explode(':',$chosen_shipping);
        if (!is_array($chosen_shipping_arr) || $chosen_shipping_arr[0] != 'local_pickup')
        {
            $delivery_url=$this->dlm_delivery_url($order);
            // if ($template_type == 'invoice')
            ?>
                <img src="<?php echo DLM_IMAGE_URI; ?>qrcode.php?code=<?php echo $delivery_url; ?>&size=<?php echo $this->qr_code_size ?>">
            <?php
        }
    }//end of function
    function dlm_links( $links, $file ) 
    {
        $fileName = 'woo-map/woo-map.php';
        
        $fileName = preg_replace('/\_trial\//', '/', $fileName);
        $fileName = preg_replace('/\_commercial\//', '/', $fileName);
        if ($file == $fileName) 
        {
            $pro_link = '<a target="_blank" href="'.admin_url('admin.php?page=wc-settings&tab=dlm_api_keys').'">'.esc_html__('Settings','dlm').'</a>';
            array_push( $links, $pro_link );          
        }
        return $links;
    }//end of function
    function dlm_add_email_order_meta( $order, $sent_to_admin, $plain_text )
    { 
        $html='';
        $delivery_url=$this->dlm_delivery_url($order);
        $lat_long=get_post_meta( $order->id, 'dlm_delivery_location', true );
        $lat_long_arr=explode(',', $lat_long);
        if ( $plain_text === false ) 
        { 
            // you shouldn't have to worry about inline styles, WooCommerce adds them itself depending on the theme you use
            $html.='<h2>Geo Location Information</h2>';
            $html.='<ul>';
                $html.='<li>';
                    $html.='<strong>Delivery Location Latitude:</strong>'.$lat_long_arr[0];
                $html.='</li>';
                $html.='<li>';
                    $html.='<strong>Delivery Location Longitude:</strong>'.$lat_long_arr[1];
                $html.='</li>';
                $html.='<li>';
                    $html.='<strong>Delivery Location Map:</strong><a href="'.$delivery_url.'">Google Map</a>';
                $html.='</li>';
            $html.='</ul>';
     
        } else {
     
            $html.="Geo Location Information\n
            Delivery Location URL:";    
        } 
        echo $html;
    }//end of function
    function dlm_goole_map_column( $columns ) {

        $new_columns = array();

        foreach ( $columns as $column_name => $column_info ) {

            $new_columns[ $column_name ] = $column_info;

            if ( 'shipping_address' === $column_name ) {
                $new_columns['google_map'] = __( 'Location', 'dlm' );
            }
        }

        return $new_columns;
    }//end of function
    function dlm_google_map_column_content( $column ) {
        global $post;

        if ( 'google_map' === $column ) {

            $order    = wc_get_order( $post->ID );
            $delivery_url=$this->dlm_delivery_url($order);
            $lat_long=get_post_meta( $order->id, 'dlm_delivery_location', true );
            $lat_long_arr=explode(',', $lat_long);
            echo '<a href="'.$delivery_url.'"><img src="'.DLM_IMAGE_URI.'street-map.png" style="width:20px;"/></a>';
        }
    }
    function dlm_checkout_map_script() 
    {
        // Only checkout page
        if( ! ( is_checkout() && ! is_wc_endpoint_url() ) ) return;
        ?>
        <script type="text/javascript">
        jQuery( function ( $ ) {
            var a = 'input[name^="shipping_method"]', b = a+':checked',
                c = 'local_pickup',
                d = '#customise_checkout_field';

            function showHideDeliveryDate()
            {
                if($(b).val())
                {
                   var shipping=$(b).val().split(':');
                   console.log( shipping);
                   $('#dlm_shipping_method').val($(b).val());
                   if( shipping[0] == c )
                   {
                       $(d).removeClass('hide');
                       $(d).hide('fast');
                   }
                   else
                   {
                       $(d).show();
                   }
                   console.log('Chosen shipping method: '+$(b).val()); // <== Just for testing (to be removed) 
                }
                
            }            
            showHideDeliveryDate();// while On loading

            // while "change" of chosen shipping method
            $('form.checkout').on('change', a, function(){
                showHideDeliveryDate();
            });
        });
        </script>
        <?php
    }//end of function
    
}//end of class
new DLM();